/*
 * DataTank-2 LED driver
 *
 * Author: John Bowler <jbowler@acm.org>
 *
 * Copyright (c) 2006 John Bowler
 *
 * Permission is hereby granted, free of charge, to any
 * person obtaining a copy of this software and associated
 * documentation files (the "Software"), to deal in the
 * Software without restriction, including without
 * limitation the rights to use, copy, modify, merge,
 * publish, distribute, sublicense, and/or sell copies of
 * the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the
 * following conditions:
 *
 * The above copyright notice and this permission notice
 * shall be included in all copies or substantial portions
 * of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF
 * ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT
 * SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR
 * ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 */

/*
 * reworked for DataTank-2 LEDS and buttons
 * by Freecom Technologies GmbH, Berlin
 */
 

#define FC_LEDPWM

#include <linux/autoconf.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/device.h>
#include <linux/spinlock.h>
#include <linux/leds.h>
#include <asm/arch/hardware.h>
#include <asm/io.h>
#include <asm/irq.h>
#include <asm/mach/irq.h>
#include <linux/timer.h>
#include <linux/delay.h>
#include <linux/slab.h>
#include <linux/interrupt.h>

#include "mvBoardEnvLib.h"
#include "mvGpp.h"

extern spinlock_t gpio_lock;

/* Up to 16 gpio lines are possible. */
#define LEDS_MAX 16
static struct dataTank2led_device {
	struct led_classdev ancestor;
	int               flags;
} dataTank2led_devices[LEDS_MAX];

static int key;
static unsigned int pressSyncJiffies, pressResetJiffies, pressPowerJiffies, pressUnplug1Jiffies, pressUnplug2Jiffies;

#ifdef CONFIG_LEDS_PWM
volatile unsigned short *ledmem_cs, led_latch;
#else
static volatile unsigned short *ledmem_cs, led_latch, led_blink_latch;
#endif

struct work_struct  hotplug_work;
static struct timer_list leds_timerlist, keys_timerlist;

#define DRIVER_NAME "LED-Class Driver for DataTank2"
#define DRIVER_VERSION "1.6"

#define DT2_GPIO_LOW		0x00
#define DT2_GPIO_HIGH		0x01

#define DT2_LED_POS_SYNC	3
#define DT2_LED_POS_POWER	2
#define DT2_LED_POS_USB1	1
#define DT2_LED_POS_USB2	0
#define DT2_LED_POS_WAN		5
#define DT2_LED_POS_WLAN	4
#define DT2_LED_POS_SATASW	6
#define DT2_LED_POS_SATAHW	7

#define DT2_LED_MASK_SYNC	(1 << DT2_LED_POS_SYNC)
#define DT2_LED_MASK_POWER	(1 << DT2_LED_POS_POWER)
#define DT2_LED_MASK_USB1	(1 << DT2_LED_POS_USB1)
#define DT2_LED_MASK_USB2	(1 << DT2_LED_POS_USB2)
#define DT2_LED_MASK_WAN	(1 << DT2_LED_POS_WAN)
#define DT2_LED_MASK_WLAN	(1 << DT2_LED_POS_WLAN)
#define DT2_LED_MASK_SATASW	(1 << DT2_LED_POS_SATASW)
#define DT2_LED_MASK_SATAHW	(1 << DT2_LED_POS_SATAHW)

#define DT2_LEDS_ALLON  	0xFF00
#define DT2_LEDS_ALLOFF 	0xFF7F
#define DT2_LEDS_STARTUP	0xFFFF



#ifdef CONFIG_HOTPLUG

/* Notify userspace when a key-event occurs,
 * by running '/sbin/hotplug key' with certain
 * environment variables set.
 */

static void key_run_sbin_hotplug(void *Action)
{
	char *argv[3], *envp[5], action_str[32];
	int i;

	sprintf(action_str, "ACTION=%s", (char *)Action);

        i = 0;
        argv[i++] = "/sbin/hotplug";
        argv[i++] = "key";
        argv[i] = NULL;

	i = 0;
	/* minimal command environment */
	envp [i++] = "HOME=/";
	envp [i++] = "PATH=/sbin:/bin:/usr/sbin:/usr/bin";
	envp [i++] = "INTERFACE=";
	envp [i++] = action_str;
	envp [i] = NULL;

        if (in_interrupt ()) {
                printk ("DT2 LED-hotplug: -- In_interrupt\n");
                return;
        }
        if (!current->fs->root) {
                printk ("DT2 LED-hotplug: -- no FS yet\n");
                return;
        }

	i=call_usermodehelper (argv[0], argv, envp, 1);
//    printk ("DT2 LED-hotplug: %s   - callusermodehelper returned %d\n",action_str, i);
}

static void leds_poll(unsigned long arg)
{
	if(led_blink_latch == 0)
		return;
	  
	*ledmem_cs = led_latch = led_latch ^ led_blink_latch;

	/* Re-arm timer */
	leds_timerlist.expires = jiffies + HZ/2;
	add_timer(&leds_timerlist);
}

static void keys_poll(unsigned long arg)
{
	int holdkey;

	if(pressPowerJiffies == 0)
		if(pressResetJiffies == 0)
			return;

//    printk ("DT2 LED-hotplug: -  POLL-in   key:%08x, jiffies:%lu, pressSyncJiffies:%u, pressResetJiffies:%u\n", key, jiffies, pressSyncJiffies, pressResetJiffies);

	if(pressPowerJiffies != 0) if(abs(jiffies - pressPowerJiffies) >= 4*HZ) {
		if (gpio_line_get(DT2_PIN_GPIO_POWER, &holdkey))
		    pressPowerJiffies = 0;
		else {
		INIT_WORK(&hotplug_work, key_run_sbin_hotplug, "SHUTDOWN");
		schedule_work(&hotplug_work);
		pressPowerJiffies = 0;
		key &= ~DT2_PIN_GPIO_POWER;
		return;
		}
	}

	if(pressResetJiffies != 0) if(abs(jiffies - pressResetJiffies) >= 4*HZ) {
		if (gpio_line_get(DT2_PIN_GPIO_RESET, &holdkey))
		    pressResetJiffies = 0;
		else {
		INIT_WORK(&hotplug_work, key_run_sbin_hotplug, "DEFAULTS");
		schedule_work(&hotplug_work);
		pressResetJiffies = 0;
		key &= ~DT2_PIN_GPIO_RESET;
		return;
		}
	}

	/* Re-arm timer */
	keys_timerlist.expires = jiffies + HZ/10;
	add_timer(&keys_timerlist);
}

static irqreturn_t leds_sync_key_interrupt(int irq, void *dev_id, struct pt_regs *regs)
{
	int holdkey;

	MV_REG_BIT_RESET(MV_GPP_IRQ_EDGE_MASK_REG, (DT2_PIN_GPIO_SYNC) );
	gpio_line_get(DT2_PIN_GPIO_SYNC, &holdkey);
//    printk ("DT2 LED-hotplug: -  SYNC-in   holdkey:%08x, key:%08x, jiffies:%lu, pressSyncJiffies:%u\n", holdkey, key, jiffies, pressSyncJiffies);
//    printk(" main cause is %08x, mask %08x \n",MV_REG_READ(MV_IRQ_CAUSE_REG), MV_REG_READ(MV_IRQ_MASK_REG) );
//    printk(" gpp cause is %08x, edge-mask %08x, level-mask %08x\n",MV_REG_READ(MV_GPP_IRQ_CAUSE_REG), MV_REG_READ(MV_GPP_IRQ_EDGE_MASK_REG), MV_REG_READ(MV_GPP_IRQ_LEVEL_MASK_REG) );
//    printk(" data in %08x, gpp out-enn %08x\n", MV_REG_READ(0x10110), MV_REG_READ(0x10104));
	if(holdkey==0) {  // key pressed ?

		if((key & DT2_PIN_GPIO_SYNC) == 0) {  // no other key still pressed ?
			keys_timerlist.expires = jiffies + HZ/10;
			add_timer(&keys_timerlist);
		}

		key |= DT2_PIN_GPIO_SYNC;
		pressSyncJiffies = jiffies;
//		mvGppPolarityInvert(0, DT2_PIN_GPIO_SYNC);
	}
	else {              // key released
//		mvGppPolarityInvert(0, DT2_PIN_GPIO_SYNC);
		key &= ~DT2_PIN_GPIO_SYNC;
//		MV_REG_BIT_RESET(MV_GPP_IRQ_CAUSE_REG, (DT2_PIN_GPIO_SYNC) );
		if(pressSyncJiffies == 0)
			goto exit;

		if(abs(jiffies - pressSyncJiffies) < 4*HZ) {
			INIT_WORK(&hotplug_work, key_run_sbin_hotplug, "USEREVENT");
			schedule_work(&hotplug_work);
		}

		pressSyncJiffies = 0;
	}
exit:
//    printk ("DT2 LED-hotplug: -  SYNC-out  holdkey:%08x, key:%08x, jiffies:%lu, pressSyncJiffies:%u\n", holdkey, key, jiffies, pressSyncJiffies);
	mvGppPolarityInvert(0, DT2_PIN_GPIO_SYNC);
	MV_REG_BIT_SET(MV_GPP_IRQ_EDGE_MASK_REG, (DT2_PIN_GPIO_SYNC) );
	MV_REG_BIT_RESET(MV_GPP_IRQ_LEVEL_MASK_REG, (DT2_PIN_GPIO_SYNC) );
	MV_REG_BIT_RESET(MV_GPP_IRQ_CAUSE_REG, (DT2_PIN_GPIO_SYNC) );
//    printk(" main cause is %08x, mask %08x \n",MV_REG_READ(MV_IRQ_CAUSE_REG), MV_REG_READ(MV_IRQ_MASK_REG) );
//    printk(" gpp cause is %08x, edge-mask %08x, level-mask %08x\n",MV_REG_READ(MV_GPP_IRQ_CAUSE_REG), MV_REG_READ(MV_GPP_IRQ_EDGE_MASK_REG), MV_REG_READ(MV_GPP_IRQ_LEVEL_MASK_REG) );
//    printk(" data in %08x, gpp out-enn %08x\n", MV_REG_READ(0x10110), MV_REG_READ(0x10104));
	return IRQ_HANDLED;
}

static irqreturn_t leds_reset_key_interrupt(int irq, void *dev_id, struct pt_regs *regs)
{
	int holdkey;

	MV_REG_BIT_RESET(MV_GPP_IRQ_EDGE_MASK_REG, (DT2_PIN_GPIO_RESET) );
	gpio_line_get(DT2_PIN_GPIO_RESET, &holdkey);
//    printk ("DT2 LED-hotplug: -  RESET-in   holdkey:%08x, key:%d, jiffies:%lu, pressResetJiffies:%u\n", holdkey, key, jiffies, pressResetJiffies);
//    printk(" main cause is %08x , mask %08x \n",MV_REG_READ(MV_IRQ_CAUSE_REG), MV_REG_READ(MV_IRQ_MASK_REG) );
//    printk(" gpp cause is %08x , mask %08x \n",MV_REG_READ(MV_GPP_IRQ_CAUSE_REG), MV_REG_READ(MV_GPP_IRQ_MASK_REG) );
//    printk(" data in %08x  gpp outen %08x \n\n", MV_REG_READ(0x10110), MV_REG_READ(0x10104));
	if(holdkey==0) {  // key pressed ?

		if((key & DT2_PIN_GPIO_RESET) == 0) {  // no other key still pressed ?
			keys_timerlist.expires = jiffies + HZ/10;
			add_timer(&keys_timerlist);
		}

		key |= DT2_PIN_GPIO_RESET;
		pressResetJiffies = jiffies;
	}
	else {              // key released
		key &= ~DT2_PIN_GPIO_RESET;
		if(pressResetJiffies == 0)
			goto exit;

		if(abs(jiffies - pressResetJiffies) < 4*HZ) {
			INIT_WORK(&hotplug_work, key_run_sbin_hotplug, "REBOOT");
			schedule_work(&hotplug_work);
		}

		pressResetJiffies = 0;
	}
exit:
//    printk ("DT2 LED-hotplug: -  RESET-out   holdkey:%08x, key:%d, jiffies:%lu, pressResetJiffies:%u\n", holdkey, key, jiffies, pressResetJiffies);
	mvGppPolarityInvert(0, DT2_PIN_GPIO_RESET);
	MV_REG_BIT_SET(MV_GPP_IRQ_EDGE_MASK_REG, (DT2_PIN_GPIO_RESET) );
	MV_REG_BIT_RESET(MV_GPP_IRQ_LEVEL_MASK_REG, (DT2_PIN_GPIO_RESET) );
	MV_REG_BIT_RESET(MV_GPP_IRQ_CAUSE_REG, (DT2_PIN_GPIO_RESET) );
//    printk(" main cause is %08x, mask %08x \n",MV_REG_READ(MV_IRQ_CAUSE_REG), MV_REG_READ(MV_IRQ_MASK_REG) );
//    printk(" gpp cause is %08x, edge-mask %08x, level-mask %08x\n",MV_REG_READ(MV_GPP_IRQ_CAUSE_REG), MV_REG_READ(MV_GPP_IRQ_EDGE_MASK_REG), MV_REG_READ(MV_GPP_IRQ_LEVEL_MASK_REG) );
//    printk(" data in %08x, gpp out-enn %08x\n", MV_REG_READ(0x10110), MV_REG_READ(0x10104));
	return IRQ_HANDLED;
}

static irqreturn_t leds_power_key_interrupt(int irq, void *dev_id, struct pt_regs *regs)
{
	int holdkey;

	MV_REG_BIT_RESET(MV_GPP_IRQ_EDGE_MASK_REG, (DT2_PIN_GPIO_POWER) );
	gpio_line_get(DT2_PIN_GPIO_POWER, &holdkey);
//    printk ("DT2 LED-hotplug: -  POWER-in   holdkey:%08x, key:%08x, jiffies:%lu, pressPowerJiffies:%u\n", holdkey, key, jiffies, pressPowerJiffies);
//    printk(" main cause is %08x , mask %08x \n",MV_REG_READ(MV_IRQ_CAUSE_REG), MV_REG_READ(MV_IRQ_MASK_REG) );
//    printk(" gpp cause is %08x , mask %08x \n",MV_REG_READ(MV_GPP_IRQ_CAUSE_REG), MV_REG_READ(MV_GPP_IRQ_MASK_REG) );
//    printk(" data in %08x  gpp outen %08x \n\n", MV_REG_READ(0x10110), MV_REG_READ(0x10104));
	if(holdkey==0) {  // key pressed ?

		if((key & DT2_PIN_GPIO_POWER) == 0) {  // no other key still pressed ?
			keys_timerlist.expires = jiffies + HZ/10;
			add_timer(&keys_timerlist);
		}

		key |= DT2_PIN_GPIO_POWER;
		pressPowerJiffies = jiffies;
	}
	else {              // key released
		key &= ~DT2_PIN_GPIO_POWER;
		if(pressPowerJiffies == 0)
			goto exit;

		if(abs(jiffies - pressPowerJiffies) < 4*HZ) {
			INIT_WORK(&hotplug_work, key_run_sbin_hotplug, "POWER");
			schedule_work(&hotplug_work);
		}

		pressPowerJiffies = 0;
	}
exit:
//    printk ("DT2 LED-hotplug: -  POWER-out   holdkey:%08x, key:%08x, jiffies:%lu, pressPowerJiffies:%u\n", holdkey, key, jiffies, pressPowerJiffies);
	mvGppPolarityInvert(0, DT2_PIN_GPIO_POWER);
	MV_REG_BIT_SET(MV_GPP_IRQ_EDGE_MASK_REG, (DT2_PIN_GPIO_POWER) );
	MV_REG_BIT_RESET(MV_GPP_IRQ_LEVEL_MASK_REG, (DT2_PIN_GPIO_POWER) );
	MV_REG_BIT_RESET(MV_GPP_IRQ_CAUSE_REG, (DT2_PIN_GPIO_POWER) );
//    printk(" main cause is %08x, mask %08x \n",MV_REG_READ(MV_IRQ_CAUSE_REG), MV_REG_READ(MV_IRQ_MASK_REG) );
//    printk(" gpp cause is %08x, edge-mask %08x, level-mask %08x\n",MV_REG_READ(MV_GPP_IRQ_CAUSE_REG), MV_REG_READ(MV_GPP_IRQ_EDGE_MASK_REG), MV_REG_READ(MV_GPP_IRQ_LEVEL_MASK_REG) );
//    printk(" data in %08x, gpp out-enn %08x\n", MV_REG_READ(0x10110), MV_REG_READ(0x10104));
	return IRQ_HANDLED;
}

static irqreturn_t leds_unplug1_key_interrupt(int irq, void *dev_id, struct pt_regs *regs)
{
	// Mechanical key are nasty, so do some pre-cautions.
//    printk ("DT2 LED-hotplug: -  USB1-in   key:%d, jiffies:%lu, pressUnplug1Jiffies:%u\n", key, jiffies, pressUnplug1Jiffies);
//    printk(" main cause is %08x , mask %08x \n",MV_REG_READ(MV_IRQ_CAUSE_REG), MV_REG_READ(MV_IRQ_MASK_REG) );
//    printk(" gpp cause is %08x , mask %08x \n",MV_REG_READ(MV_GPP_IRQ_CAUSE_REG), MV_REG_READ(MV_GPP_IRQ_MASK_REG) );
//    printk(" data in %08x  gpp outen %08x \n\n", MV_REG_READ(0x10110), MV_REG_READ(0x10104));
	MV_REG_BIT_RESET(MV_GPP_IRQ_EDGE_MASK_REG, (DT2_PIN_GPIO_UNPLUG1) );
	if(abs(jiffies - pressUnplug1Jiffies) < HZ/2) {
		pressUnplug1Jiffies = jiffies;
		goto exit;
	}
	pressUnplug1Jiffies = jiffies;

	if((led_latch & DT2_LED_MASK_USB1) == 0) {   // test USB1 LED
		INIT_WORK(&hotplug_work, key_run_sbin_hotplug, "UNPLUG USB1");
		schedule_work(&hotplug_work);
	}
//	else {
//		INIT_WORK(&hotplug_work, key_run_sbin_hotplug, "COLDPLUG USB1");
//		schedule_work(&hotplug_work);
//	}

exit:
	MV_REG_BIT_SET(MV_GPP_IRQ_EDGE_MASK_REG, (DT2_PIN_GPIO_UNPLUG1) );
	MV_REG_BIT_RESET(MV_GPP_IRQ_LEVEL_MASK_REG, (DT2_PIN_GPIO_UNPLUG1) );
	MV_REG_BIT_RESET(MV_GPP_IRQ_CAUSE_REG, (DT2_PIN_GPIO_UNPLUG1) );
//    printk(" main cause is %08x, mask %08x \n",MV_REG_READ(MV_IRQ_CAUSE_REG), MV_REG_READ(MV_IRQ_MASK_REG) );
//    printk(" gpp cause is %08x, edge-mask %08x, level-mask %08x\n",MV_REG_READ(MV_GPP_IRQ_CAUSE_REG), MV_REG_READ(MV_GPP_IRQ_EDGE_MASK_REG), MV_REG_READ(MV_GPP_IRQ_LEVEL_MASK_REG) );
//    printk(" data in %08x, gpp out-enn %08x\n", MV_REG_READ(0x10110), MV_REG_READ(0x10104));
	return IRQ_HANDLED;
}

static irqreturn_t leds_unplug2_key_interrupt(int irq, void *dev_id, struct pt_regs *regs)
{
	// Mechanical key are nasty, so do some pre-cautions.
//    printk ("DT2 LED-hotplug: -  USB2-in   key:%d, jiffies:%lu, pressUnplug2Jiffies:%u\n", key, jiffies, pressUnplug2Jiffies);
//    printk(" main cause is %08x , mask %08x \n",MV_REG_READ(MV_IRQ_CAUSE_REG), MV_REG_READ(MV_IRQ_MASK_REG) );
//    printk(" gpp cause is %08x , mask %08x \n",MV_REG_READ(MV_GPP_IRQ_CAUSE_REG), MV_REG_READ(MV_GPP_IRQ_MASK_REG) );
//    printk(" data in %08x  gpp outen %08x \n\n", MV_REG_READ(0x10110), MV_REG_READ(0x10104));
	MV_REG_BIT_RESET(MV_GPP_IRQ_EDGE_MASK_REG, (DT2_PIN_GPIO_UNPLUG2) );
	if(abs(jiffies - pressUnplug2Jiffies) < HZ/2) {
		pressUnplug2Jiffies = jiffies;
		goto exit;
	}
	pressUnplug2Jiffies = jiffies;

	if((led_latch & DT2_LED_MASK_USB2) == 0) {   // test USB2 LED
		INIT_WORK(&hotplug_work, key_run_sbin_hotplug, "UNPLUG USB2");
		schedule_work(&hotplug_work);
	}
//	else {
//		INIT_WORK(&hotplug_work, key_run_sbin_hotplug, "COLDPLUG USB2");
//		schedule_work(&hotplug_work);
//	}

exit:
	MV_REG_BIT_SET(MV_GPP_IRQ_EDGE_MASK_REG, (DT2_PIN_GPIO_UNPLUG2) );
	MV_REG_BIT_RESET(MV_GPP_IRQ_LEVEL_MASK_REG, (DT2_PIN_GPIO_UNPLUG2) );
	MV_REG_BIT_RESET(MV_GPP_IRQ_CAUSE_REG, (DT2_PIN_GPIO_UNPLUG2) );
//    printk(" main cause is %08x, mask %08x \n",MV_REG_READ(MV_IRQ_CAUSE_REG), MV_REG_READ(MV_IRQ_MASK_REG) );
//    printk(" gpp cause is %08x, edge-mask %08x, level-mask %08x\n",MV_REG_READ(MV_GPP_IRQ_CAUSE_REG), MV_REG_READ(MV_GPP_IRQ_EDGE_MASK_REG), MV_REG_READ(MV_GPP_IRQ_LEVEL_MASK_REG) );
//    printk(" data in %08x, gpp out-enn %08x\n", MV_REG_READ(0x10110), MV_REG_READ(0x10104));
	return IRQ_HANDLED;
}
#endif   /* CONFIG_HOTPLUG */

void dataTank2led_all_off(void)
{
	// all LEDs off and EL ring always off
	*ledmem_cs = led_latch = DT2_LEDS_ALLOFF;
}

void dataTank2led_brightness_set(struct led_classdev *pled, enum led_brightness value)
{
#ifdef CONFIG_LEDS_PWM
	extern unsigned int led_pwm0_3, led_pwm4_7;
#endif
	const struct dataTank2led_device *const dataTank2_dev =
		container_of(pled, struct dataTank2led_device, ancestor);
	const u32 led_no = dataTank2_dev - dataTank2led_devices;

	if (led_no < LEDS_MAX && dataTank2_dev->ancestor.name != 0) {
#ifdef CONFIG_LEDS_PWM
//	printk("LEDs driver - bright-set (enter) for LED-Mask:%02x with %02x, - led_latch=%04x, - ledmem_cs=&%p, - led_pwm0_3=%08x, led_pwm4_7=%08x\n", dataTank2_dev->flags, value, led_latch, ledmem_cs, led_pwm0_3, led_pwm4_7);
		if(value == 0)
			led_latch = led_latch | (dataTank2_dev->flags);
		else
			led_latch = led_latch & (~dataTank2_dev->flags);

		if(value > CONFIG_LEDS_PWM_STEPS)
			value = 0;
		else
			value = CONFIG_LEDS_PWM_STEPS - value;

		switch (dataTank2_dev->flags)
		{
		    case 0x01:
			led_pwm0_3 &= ~(0xFF << 0);
			led_pwm0_3 |= (value << 0);
			break;
		    case 0x02:
			led_pwm0_3 &= ~(0xFF << 8);
			led_pwm0_3 |= (value << 8);
			break;
		    case 0x04:
			led_pwm0_3 &= ~(0xFF << 16);
			led_pwm0_3 |= (value << 16);
			break;
		    case 0x08:
			led_pwm0_3 &= ~(0xFF << 24);
			led_pwm0_3 |= (value << 24);
			break;
		    case 0x10:
			led_pwm4_7 &= ~(0xFF << 0);
			led_pwm4_7 |= (value << 0);
			break;
		    case 0x20:
			led_pwm4_7 &= ~(0xFF << 8);
			led_pwm4_7 |= (value << 8);
			break;
		    case 0x40:
			led_pwm4_7 &= ~(0xFF << 16);
			led_pwm4_7 |= (value << 16);
			break;
		    case 0x80:
			led_pwm4_7 &= ~(0xFF << 24);
			led_pwm4_7 |= (value << 24);
			break;
		    default:
			printk("LEDs driver - bright-set PWM-value error (should never happen\n");
		}
//	printk("LEDs driver - bright-set (leave) for LED-Mask:%02x with %02x, - led_latch=%04x, - ledmem_cs=&%p, - led_pwm0_3=%08x, led_pwm4_7=%08x\n", dataTank2_dev->flags, value, led_latch, ledmem_cs, led_pwm0_3, led_pwm4_7);
#elif !defined(NEW_BLINK)
		if(value == 2) {
			if(led_blink_latch == 0) {
				leds_timerlist.expires = jiffies + HZ/2;
				add_timer(&leds_timerlist);
			}
			led_blink_latch |= dataTank2_dev->flags;
			*ledmem_cs = led_latch = led_latch & (~dataTank2_dev->flags);
		}
		else {
			led_blink_latch &= (~dataTank2_dev->flags);
			if(value == 0)
				*ledmem_cs = led_latch = led_latch | dataTank2_dev->flags;
			else
				*ledmem_cs = led_latch = led_latch & (~dataTank2_dev->flags);
		}
#else
		if(value == 0)
			*ledmem_cs = led_latch = led_latch | (dataTank2_dev->flags);
		else
			*ledmem_cs = led_latch = led_latch & (~dataTank2_dev->flags);
#endif
	}
}

/* LEDs are described in resources, the following iterates over the valid
 * LED resources.
 */
#define for_all_leds(i, pdev) \
	for (i=0; i<pdev->num_resources; ++i) \
		if (pdev->resource[i].start < LEDS_MAX && \
			pdev->resource[i].name != 0)

/* The following applies 'operation' to each LED from the given platform,
 * the function always returns 0 to allow tail call elimination.
 */
static int apply_to_all_leds(struct platform_device *pdev,
	void (*operation)(struct led_classdev *pled))
{
	int i;

	for_all_leds(i, pdev)
		operation(&dataTank2led_devices[pdev->resource[i].start].ancestor);
	return 0;
}

#ifdef CONFIG_PM
static int dataTank2led_suspend(struct platform_device *pdev, pm_message_t state)
{
	printk("LEDs driver - suspend -\n");

	return apply_to_all_leds(pdev, led_classdev_suspend);
}

static int dataTank2led_resume(struct platform_device *pdev)
{
	printk("LEDs driver - resume -\n");

	return apply_to_all_leds(pdev, led_classdev_resume);
}
#endif

static void dataTank2led_remove_one_led(struct led_classdev *pled)
{
	led_classdev_unregister(pled);
	pled->name = 0;
}

static int dataTank2led_remove(struct device *_dev)
{
	struct platform_device  *pdev = to_platform_device(_dev);
	
	return apply_to_all_leds(pdev, dataTank2led_remove_one_led);
}

static int dataTank2led_probe(struct device *_dev)
{
	/* The board level has to tell the driver where the
	 * LEDs are connected - there is no way to find out
	 * electrically.  It must also say whether the GPIO
	 * lines are active high or active low.
	 *
	 * To do this read the num_resources (the number of
	 * LEDs) and the struct resource (the data for each
	 * LED).  The name comes from the resource, and it
	 * isn't copied.
	 */
	int i;
	struct platform_device  *pdev = to_platform_device(_dev);

	printk(DRIVER_NAME " - Version:" DRIVER_VERSION "\n");

	/* Request the LED chip select address space */
	ledmem_cs = (volatile unsigned short *) mvBoardGetDeviceBaseAddr(0, BOARD_DEV_DISPLAY_LED);
	*ledmem_cs = led_latch = DT2_LEDS_STARTUP;

	led_blink_latch = DT2_LED_MASK_POWER;    // power LED blinking

#ifdef CONFIG_HOTPLUG
//    key_run_sbin_hotplug("PROBING");
	/* Configure interrupt input for SYNC switch */
	set_irq_handler(DT2_IRQ_GPIO_SYNC, do_edge_IRQ);
	set_irq_type(DT2_IRQ_GPIO_SYNC, IRQT_BOTHEDGE);
	MV_REG_BIT_RESET(MV_GPP_IRQ_CAUSE_REG, (DT2_PIN_GPIO_SYNC) );
	if (request_irq(DT2_IRQ_GPIO_SYNC, leds_sync_key_interrupt, SA_INTERRUPT, "Sync", NULL) < 0)
		printk("LEDs: failed to register IRQ%d for SYNC switch\n", DT2_IRQ_GPIO_SYNC);
	else
		printk("LEDs: registered SYNC switch on IRQ%d\n", DT2_IRQ_GPIO_SYNC);
	MV_REG_BIT_SET(MV_GPP_IRQ_EDGE_MASK_REG, (DT2_PIN_GPIO_SYNC) );
//    printk(" main cause is %08x, mask %08x \n",MV_REG_READ(MV_IRQ_CAUSE_REG), MV_REG_READ(MV_IRQ_MASK_REG) );
//    printk(" gpp cause is %08x, edge-mask %08x, level-mask %08x\n",MV_REG_READ(MV_GPP_IRQ_CAUSE_REG), MV_REG_READ(MV_GPP_IRQ_EDGE_MASK_REG), MV_REG_READ(MV_GPP_IRQ_LEVEL_MASK_REG) );
//    printk(" data in %08x, gpp out-enn %08x\n", MV_REG_READ(0x10110), MV_REG_READ(0x10104));

	/* Configure interrupt input for RESET switch */
	set_irq_handler(DT2_IRQ_GPIO_RESET, do_edge_IRQ);
	set_irq_type(DT2_IRQ_GPIO_RESET, IRQT_BOTHEDGE);
	MV_REG_BIT_RESET(MV_GPP_IRQ_CAUSE_REG, (DT2_PIN_GPIO_RESET) );
	if (request_irq(DT2_IRQ_GPIO_RESET, leds_reset_key_interrupt, SA_INTERRUPT, "Reset", NULL) < 0)
		printk("LEDs: failed to register IRQ%d for RESET switch\n", DT2_IRQ_GPIO_RESET);
	else
		printk("LEDs: registered RESET switch on IRQ%d\n", DT2_IRQ_GPIO_RESET);
//    printk(" main cause is %08x, mask %08x \n",MV_REG_READ(MV_IRQ_CAUSE_REG), MV_REG_READ(MV_IRQ_MASK_REG) );
//    printk(" gpp cause is %08x, edge-mask %08x, level-mask %08x\n",MV_REG_READ(MV_GPP_IRQ_CAUSE_REG), MV_REG_READ(MV_GPP_IRQ_EDGE_MASK_REG), MV_REG_READ(MV_GPP_IRQ_LEVEL_MASK_REG) );
//    printk(" data in %08x, gpp out-enn %08x\n", MV_REG_READ(0x10110), MV_REG_READ(0x10104));

	/* Configure interrupt input for POWER switch */
	set_irq_handler(DT2_IRQ_GPIO_POWER, do_edge_IRQ);
	set_irq_type(DT2_IRQ_GPIO_POWER, IRQT_BOTHEDGE);
	MV_REG_BIT_RESET(MV_GPP_IRQ_CAUSE_REG, (DT2_PIN_GPIO_POWER) );
	if (request_irq(DT2_IRQ_GPIO_POWER, leds_power_key_interrupt, SA_INTERRUPT, "Power", NULL) < 0)
		printk("LEDs: failed to register IRQ%d for POWER switch\n", DT2_IRQ_GPIO_POWER);
	else
		printk("LEDs: registered POWER switch on IRQ%d\n", DT2_IRQ_GPIO_POWER);
//    printk(" main cause is %08x, mask %08x \n",MV_REG_READ(MV_IRQ_CAUSE_REG), MV_REG_READ(MV_IRQ_MASK_REG) );
//    printk(" gpp cause is %08x, edge-mask %08x, level-mask %08x\n",MV_REG_READ(MV_GPP_IRQ_CAUSE_REG), MV_REG_READ(MV_GPP_IRQ_EDGE_MASK_REG), MV_REG_READ(MV_GPP_IRQ_LEVEL_MASK_REG) );
//    printk(" data in %08x, gpp out-enn %08x\n", MV_REG_READ(0x10110), MV_REG_READ(0x10104));

	/* Configure interrupt input for UNPLUG1 switch */
	set_irq_handler(DT2_IRQ_GPIO_UNPLUG1, do_edge_IRQ);
	set_irq_type(DT2_IRQ_GPIO_UNPLUG1, IRQT_FALLING);
	MV_REG_BIT_RESET(MV_GPP_IRQ_CAUSE_REG, (DT2_PIN_GPIO_UNPLUG1) );
	if (request_irq(DT2_IRQ_GPIO_UNPLUG1, leds_unplug1_key_interrupt, SA_INTERRUPT, "Unplug1", NULL) < 0)
		printk("LEDs: failed to register IRQ%d for UNPLUG-1 switch\n", DT2_IRQ_GPIO_UNPLUG1);
	else
		printk("LEDs: registered UNPLUG-1 switch on IRQ%d\n", DT2_IRQ_GPIO_UNPLUG1);
//    printk(" main cause is %08x, mask %08x \n",MV_REG_READ(MV_IRQ_CAUSE_REG), MV_REG_READ(MV_IRQ_MASK_REG) );
//    printk(" gpp cause is %08x, edge-mask %08x, level-mask %08x\n",MV_REG_READ(MV_GPP_IRQ_CAUSE_REG), MV_REG_READ(MV_GPP_IRQ_EDGE_MASK_REG), MV_REG_READ(MV_GPP_IRQ_LEVEL_MASK_REG) );
//    printk(" data in %08x, gpp out-enn %08x\n", MV_REG_READ(0x10110), MV_REG_READ(0x10104));

	/* Configure interrupt input for UNPLUG2 switch */
	set_irq_handler(DT2_IRQ_GPIO_UNPLUG2, do_edge_IRQ);
	set_irq_type(DT2_IRQ_GPIO_UNPLUG2, IRQT_FALLING);
	MV_REG_BIT_RESET(MV_GPP_IRQ_CAUSE_REG, (DT2_PIN_GPIO_UNPLUG2) );
	MV_REG_WRITE(MV_GPP_IRQ_CAUSE_REG, 0 );
	if (request_irq(DT2_IRQ_GPIO_UNPLUG2, leds_unplug2_key_interrupt, SA_INTERRUPT, "Unplug2", NULL) < 0)
		printk("LEDs: failed to register IRQ%d for UNPLUG-2 switch\n", DT2_IRQ_GPIO_UNPLUG2);
	else
		printk("LEDs: registered UNPLUG-2 switch on IRQ%d\n", DT2_IRQ_GPIO_UNPLUG2);
//    printk(" main cause is %08x, mask %08x \n",MV_REG_READ(MV_IRQ_CAUSE_REG), MV_REG_READ(MV_IRQ_MASK_REG) );
//    printk(" gpp cause is %08x, edge-mask %08x, level-mask %08x\n",MV_REG_READ(MV_GPP_IRQ_CAUSE_REG), MV_REG_READ(MV_GPP_IRQ_EDGE_MASK_REG), MV_REG_READ(MV_GPP_IRQ_LEVEL_MASK_REG) );
//    printk(" data in %08x, gpp out-enn %08x\n", MV_REG_READ(0x10110), MV_REG_READ(0x10104));
#endif

	init_timer(&leds_timerlist);
	leds_timerlist.expires = jiffies + HZ/2;
	leds_timerlist.function = leds_poll;
	leds_timerlist.data = 0;
	add_timer(&leds_timerlist);

#ifdef CONFIG_HOTPLUG
	init_timer(&keys_timerlist);
	keys_timerlist.expires = jiffies + HZ/2;
	keys_timerlist.function = keys_poll;
	keys_timerlist.data = 0;
	add_timer(&keys_timerlist);

	// at start-up one IRQ is released !?! So reset everything.
	key = 0;
	pressSyncJiffies = pressResetJiffies = pressPowerJiffies = pressUnplug1Jiffies = pressUnplug2Jiffies = 0;
#endif


	for_all_leds(i, pdev) {
		const u8 gpio_pin = pdev->resource[i].start;
		int      rc;

		if (dataTank2led_devices[gpio_pin].ancestor.name == 0) {

			dataTank2led_devices[gpio_pin].flags =
				pdev->resource[i].flags & IORESOURCE_BITS;

			dataTank2led_devices[gpio_pin].ancestor.name =
				pdev->resource[i].name;

			/* This is how a board manufacturer makes the LED
			 * come on on reset - the GPIO line will be high, so
			 * make the LED light when the line is low...
			 */
			if (dataTank2led_devices[gpio_pin].flags != DT2_GPIO_LOW)
				dataTank2led_devices[gpio_pin].ancestor.brightness = 100;
			else
				dataTank2led_devices[gpio_pin].ancestor.brightness = 0;

			dataTank2led_devices[gpio_pin].ancestor.flags = 0;

			dataTank2led_devices[gpio_pin].ancestor.brightness_set =
				dataTank2led_brightness_set;

			dataTank2led_devices[gpio_pin].ancestor.default_trigger = 0;
		}

		rc = led_classdev_register(&pdev->dev,
				&dataTank2led_devices[gpio_pin].ancestor);
		if (rc < 0) {
			dataTank2led_devices[gpio_pin].ancestor.name = 0;
			dataTank2led_remove(_dev);
			return rc;
		}
	}

	return 0;
}

static struct resource dataTank2_led_resources[] = {
	{
		.name		= "SYNC",
		.start		= DT2_LED_POS_SYNC,
		.end		= DT2_LED_POS_SYNC,
		.flags		= DT2_LED_MASK_SYNC,
	},
	{
		.name		= "POWER",
		.start		= DT2_LED_POS_POWER,
		.end		= DT2_LED_POS_POWER,
		.flags		= DT2_LED_MASK_POWER,
	},
	{
		.name		= "USB1",
		.start		= DT2_LED_POS_USB1,
		.end		= DT2_LED_POS_USB1,
		.flags		= DT2_LED_MASK_USB1,
	},
	{
		.name		= "USB2",
		.start		= DT2_LED_POS_USB2,
		.end		= DT2_LED_POS_USB2,
		.flags		= DT2_LED_MASK_USB2,
	},
	{
		.name		= "WAN",
		.start		= DT2_LED_POS_WAN,
		.end		= DT2_LED_POS_WAN,
		.flags		= DT2_LED_MASK_WAN,
	},
	{
		.name		= "WLAN",
		.start		= DT2_LED_POS_WLAN,
		.end		= DT2_LED_POS_WLAN,
		.flags		= DT2_LED_MASK_WLAN,
	},
	{
		.name		= "SATA_SW",
		.start		= DT2_LED_POS_SATASW,
		.end		= DT2_LED_POS_SATASW,
		.flags		= DT2_LED_MASK_SATASW,
	},
	{
		.name		= "SATA_HW",
		.start		= DT2_LED_POS_SATAHW,
		.end		= DT2_LED_POS_SATAHW,
		.flags		= DT2_LED_MASK_SATAHW,
	},
};

static struct platform_device dataTank2_leds = {
	.name			= "DT2-MEMIO-LED",
	.id			= -1,
	.num_resources		= ARRAY_SIZE(dataTank2_led_resources),
	.resource		= dataTank2_led_resources,
};

static struct device_driver dataTank2led_driver = {
	.name    = "DT2-MEMIO-LED",
        .bus     = &platform_bus_type,
	.probe   = dataTank2led_probe,
	.remove  = dataTank2led_remove,
#ifdef CONFIG_PM
	.suspend = dataTank2led_suspend,
	.resume  = dataTank2led_resume,
#endif
};

static int __init dataTank2led_init(void)
{
	driver_register(&dataTank2led_driver);
	platform_device_register(&dataTank2_leds);

	return 0;
}

static void __exit dataTank2led_exit(void)
{
	free_irq(DT2_IRQ_GPIO_SYNC, NULL);
	free_irq(DT2_IRQ_GPIO_RESET, NULL);
	free_irq(DT2_IRQ_GPIO_POWER, NULL);
	free_irq(DT2_IRQ_GPIO_RESET, NULL);
	free_irq(DT2_IRQ_GPIO_UNPLUG2, NULL);

	platform_device_unregister(&dataTank2_leds);
	driver_unregister(&dataTank2led_driver);
}

module_init(dataTank2led_init);
module_exit(dataTank2led_exit);

MODULE_AUTHOR("Freecom GmbH - Based on source by John Bowler <jbowler@acm.org>");
MODULE_DESCRIPTION("DataTank MEM-IO LED driver");
MODULE_LICENSE("Dual MIT/GPL");
